@@ -1,15 +1,18 @@
 import calendar
 from collections import defaultdict
 from datetime import datetime
-from time import sleep, time
+from time import sleep
+from time import time
 from hashlib import sha256
 import json
+import logging
 
 from apiclient.discovery import build
 from apiclient.errors import HttpError
 import httplib2
 
-from bigquery import logger
+from bigquery.errors import JobExecutingException
+from bigquery.errors import JobInsertException
 from bigquery.errors import UnfinishedQueryException
 from bigquery.schema_builder import schema_from_record
 
@@ -19,18 +22,29 @@ BIGQUERY_SCOPE_READ_ONLY = 'https://www.googleapis.com/auth/bigquery.readonly'
 
 JOB_CREATE_IF_NEEDED = 'CREATE_IF_NEEDED'
 JOB_CREATE_NEVER = 'CREATE_NEVER'
-JOB_SOURCE_FORMAT_NEWLINE_DELIMITED_JSON = 'NEWLINE_DELIMITED_JSON'
-JOB_SOURCE_FORMAT_DATASTORE_BACKUP = 'DATASTORE_BACKUP'
-JOB_SOURCE_FORMAT_CSV = 'CSV'
 JOB_WRITE_TRUNCATE = 'WRITE_TRUNCATE'
 JOB_WRITE_APPEND = 'WRITE_APPEND'
 JOB_WRITE_EMPTY = 'WRITE_EMPTY'
 JOB_ENCODING_UTF_8 = 'UTF-8'
 JOB_ENCODING_ISO_8859_1 = 'ISO-8859-1'
+JOB_PRIORITY_INTERACTIVE = 'INTERACTIVE'
+JOB_PRIORITY_BATCH = 'BATCH'
+JOB_COMPRESSION_NONE = 'NONE'
+JOB_COMPRESSION_GZIP = 'GZIP'
+
+JOB_FORMAT_CSV = 'CSV'
+JOB_FORMAT_NEWLINE_DELIMITED_JSON = 'NEWLINE_DELIMITED_JSON'
+JOB_SOURCE_FORMAT_DATASTORE_BACKUP = 'DATASTORE_BACKUP'
+JOB_SOURCE_FORMAT_NEWLINE_DELIMITED_JSON = JOB_FORMAT_NEWLINE_DELIMITED_JSON
+JOB_SOURCE_FORMAT_CSV = JOB_FORMAT_CSV
+JOB_DESTINATION_FORMAT_AVRO = 'AVRO'
+JOB_DESTINATION_FORMAT_NEWLINE_DELIMITED_JSON = \
+    JOB_FORMAT_NEWLINE_DELIMITED_JSON
+JOB_DESTINATION_FORMAT_CSV = JOB_FORMAT_CSV
 
 
 def get_client(project_id, credentials=None, service_account=None,
-               private_key=None, readonly=True):
+               private_key=None, readonly=True, swallow_results=True):
     """Return a singleton instance of BigQueryClient. Either
     AssertionCredentials or a service account and private key combination need
     to be provided in order to authenticate requests to BigQuery.
@@ -44,6 +58,8 @@ def get_client(project_id, credentials=None, service_account=None,
                      PKCS12 or PEM format.
         readonly: bool indicating if BigQuery access is read-only. Has no
                   effect if credentials are provided.
+        swallow_results: If set to false then return the actual response value
+                         instead of converting to a boolean.
 
     Returns:
         an instance of BigQueryClient.
@@ -58,7 +74,7 @@ def get_client(project_id, credentials=None, service_account=None,
                                  private_key=private_key,
                                  readonly=readonly)
 
-    return BigQueryClient(bq_service, project_id)
+    return BigQueryClient(bq_service, project_id, swallow_results)
 
 
 def _get_bq_service(credentials=None, service_account=None, private_key=None,
@@ -86,9 +102,10 @@ def _credentials():
 
 
 class BigQueryClient(object):
-    def __init__(self, bq_service, project_id):
+    def __init__(self, bq_service, project_id, swallow_results=True):
         self.bigquery = bq_service
         self.project_id = project_id
+        self.swallow_results = swallow_results
 
     def query(self, query, max_results=None, timeout=10, dry_run=False):
         """Submit a query to BigQuery.
@@ -108,7 +125,7 @@ class BigQueryClient(object):
             or a dict containing the response if invalid.
         """
 
-        logger.debug('Executing query: %s' % query)
+        logging.debug('Executing query: %s' % query)
 
         job_collection = self.bigquery.jobs()
         query_data = {
@@ -121,7 +138,7 @@ class BigQueryClient(object):
         try:
             query_reply = job_collection.query(
                 projectId=self.project_id, body=query_data).execute()
-        except HttpError, e:
+        except HttpError as e:
             if dry_run:
                 return None, json.loads(e.content)
             raise
@@ -146,7 +163,7 @@ class BigQueryClient(object):
             job_collection, self.project_id, job_id, offset=0, limit=0)
 
         if not query_reply['jobComplete']:
-            logger.warning('BigQuery job %s not complete' % job_id)
+            logging.warning('BigQuery job %s not complete' % job_id)
             raise UnfinishedQueryException()
 
         return query_reply['schema']['fields']
@@ -170,7 +187,7 @@ class BigQueryClient(object):
                 datasetId=dataset).execute()
         except HttpError, e:
             if int(e.resp['status']) == 404:
-                logger.warn('Table %s.%s does not exist', dataset, table)
+                logging.warn('Table %s.%s does not exist', dataset, table)
                 return None
             raise
 
@@ -192,7 +209,7 @@ class BigQueryClient(object):
             job_collection, self.project_id, job_id, offset=0, limit=0)
 
         return (query_reply.get('jobComplete', False),
-                query_reply.get('totalRows', 0))
+                int(query_reply.get('totalRows', 0)))
 
     def get_query_rows(self, job_id, offset=None, limit=None):
         """Retrieve a list of rows from a query table by job id.
@@ -212,7 +229,7 @@ class BigQueryClient(object):
             limit=limit)
 
         if not query_reply['jobComplete']:
-            logger.warning('BigQuery job %s not complete' % job_id)
+            logging.warning('BigQuery job %s not complete' % job_id)
             raise UnfinishedQueryException()
 
         schema = query_reply['schema']['fields']
@@ -229,16 +246,30 @@ class BigQueryClient(object):
 
         Returns:
             bool indicating if the table exists.
-    """
+        """
+        table = self.get_table(dataset, table)
+        return bool(table)
 
+    def get_table(self, dataset, table):
+        """
+        Retrieve a table if it exists, otherwise return an empty dict.
+
+        Args:
+            dataset: the dataset that the table is in
+            table: the name of the table
+
+        Returns:
+            dictionary containing the table object if it exists, otherwise
+            an empty dictionary
+        """
         try:
-            self.bigquery.tables().get(
+            table = self.bigquery.tables().get(
                 projectId=self.project_id, datasetId=dataset,
                 tableId=table).execute()
-            return True
+        except HttpError:
+            table = {}
 
-        except:
-            return False
+        return table
 
     def create_table(self, dataset, table, schema):
         """Create a new table in the dataset.
@@ -249,7 +280,8 @@ class BigQueryClient(object):
             schema: table schema dict.
 
         Returns:
-            bool indicating if the table was successfully created or not.
+            bool indicating if the table was successfully created or not,
+            or response from BigQuery if swallow_results is set for False.
         """
 
         body = {
@@ -262,16 +294,24 @@ class BigQueryClient(object):
         }
 
         try:
-            self.bigquery.tables().insert(
+            table = self.bigquery.tables().insert(
                 projectId=self.project_id,
                 datasetId=dataset,
                 body=body
             ).execute()
-            return True
+            if self.swallow_results:
+                return True
+            else:
+                return table
 
-        except:
-            logger.error('Cannot create table %s.%s' % (dataset, table))
-            return False
+        except HttpError as e:
+            logging.error(('Cannot create table {0}.{1}\n'
+                           'Http Error: {2}').format(dataset, table,
+                                                     e.message))
+            if self.swallow_results:
+                return False
+            else:
+                return {}
 
     def delete_table(self, dataset, table):
         """Delete a table from the dataset.
@@ -281,20 +321,29 @@ class BigQueryClient(object):
             table: the name of the table to delete.
 
         Returns:
-            bool indicating if the table was successfully deleted or not.
+            bool indicating if the table was successfully deleted or not,
+            or response from BigQuery if swallow_results is set for False.
         """
 
         try:
-            self.bigquery.tables().delete(
+            response = self.bigquery.tables().delete(
                 projectId=self.project_id,
                 datasetId=dataset,
                 tableId=table
             ).execute()
-            return True
+            if self.swallow_results:
+                return True
+            else:
+                return response
 
-        except:
-            logger.error('Cannot delete table %s.%s' % (dataset, table))
-            return False
+        except HttpError as e:
+            logging.error(('Cannot delete table {0}.{1}\n'
+                           'Http Error: {2}').format(dataset, table,
+                                                     e.message))
+            if self.swallow_results:
+                return False
+            else:
+                return {}
 
     def get_tables(self, dataset_id, app_id, start_time, end_time):
         """Retrieve a list of tables that are related to the given app id
@@ -374,7 +423,9 @@ class BigQueryClient(object):
             https://developers.google.com/bigquery/docs/reference/v2/jobs
 
         Returns:
-            dict, a BigQuery job resource or None on failure
+            dict, a BigQuery job resource
+        Raises:
+            JobInsertException on http/auth failures or error in result
         """
         source_uris = source_uris if isinstance(source_uris, list) \
             else [source_uris]
@@ -410,7 +461,7 @@ class BigQueryClient(object):
             configuration['sourceFormat'] = source_format
 
         if not job:
-            hex = sha256(":".join(source_uris) + str(time())).hexdigest()
+            hex = self._generate_hex_for_uris(source_uris)
             job = "{dataset}-{table}-{digest}".format(
                 dataset=dataset,
                 table=table,
@@ -459,37 +510,192 @@ class BigQueryClient(object):
             }
         }
 
-        try:
-            logger.info("Creating load job %s" % body)
-            job_resource = self.bigquery.jobs() \
-                .insert(projectId=self.project_id, body=body) \
-                .execute()
-            return job_resource
-        except Exception, e:
-            logger.error("Failed while starting uri import job: {0}"
-                         .format(e))
-            return None
+        logging.debug("Creating load job %s" % body)
+        job_resource = self.bigquery.jobs() \
+            .insert(projectId=self.project_id, body=body) \
+            .execute()
+        self._raise_insert_exception_if_error(job_resource)
+        return job_resource
+
+    def export_data_to_uris(
+            self,
+            destination_uris,
+            dataset,
+            table,
+            job=None,
+            compression=None,
+            destination_format=None,
+            print_header=None,
+            field_delimiter=None,
+    ):
+        """
+        Export data from a BigQuery table to cloud storage.
+        Args:
+            destination_uris: required string or list of strings representing
+                              the uris on cloud storage of the form:
+                              gs://bucket/filename
+            dataset: required string id of the dataset
+            table: required string id of the table
+            job: optional string identifying the job (a unique jobid
+                    is automatically generated if not provided)
+            compression: optional string
+                    (one of the JOB_COMPRESSION_* constants)
+            destination_format: optional string
+                    (one of the JOB_DESTINATION_FORMAT_* constants)
+            print_header: optional boolean
+            field_delimiter: optional string
+
+            Optional arguments with value None are determined by
+            BigQuery as described:
+            https://developers.google.com/bigquery/docs/reference/v2/jobs
+
+        Returns:
+            dict, a BigQuery job resource
+        Raises:
+            JobInsertException on http/auth failures or error in result
+        """
+        destination_uris = destination_uris \
+            if isinstance(destination_uris, list) else [destination_uris]
+
+        configuration = {
+            "sourceTable": {
+                "projectId": self.project_id,
+                "tableId": table,
+                "datasetId": dataset
+            },
+            "destinationUris": destination_uris,
+        }
+
+        if compression:
+            configuration['compression'] = compression
+
+        if destination_format:
+            configuration['destinationFormat'] = destination_format
+
+        if print_header is not None:
+            configuration['printHeader'] = print_header
+
+        if field_delimiter:
+            configuration['fieldDelimiter'] = field_delimiter
+
+        if not job:
+            hex = self._generate_hex_for_uris(destination_uris)
+            job = "{dataset}-{table}-{digest}".format(
+                dataset=dataset,
+                table=table,
+                digest=hex
+            )
+
+        body = {
+            "configuration": {
+                'extract': configuration
+            },
+            "jobReference": {
+                "projectId": self.project_id,
+                "jobId": job
+            }
+        }
+
+        logging.info("Creating export job %s" % body)
+        job_resource = self.bigquery.jobs() \
+            .insert(projectId=self.project_id, body=body) \
+            .execute()
+        self._raise_insert_exception_if_error(job_resource)
+        return job_resource
+
+    def write_to_table(
+            self,
+            query,
+            dataset=None,
+            table=None,
+            allow_large_results=None,
+            use_query_cache=None,
+            priority=None,
+            create_disposition=None,
+            write_disposition=None,
+    ):
+        """
+        Write query result to table. If dataset or table is not provided,
+        Bigquery will write the result to temporary table.
+        Args:
+            query: required BigQuery query string.
+            dataset: optional string id of the dataset
+            table: optional string id of the table
+            allow_large_results: optional boolean
+            use_query_cache: optional boolean
+            priority: optional string
+                    (one of the JOB_PRIORITY_* constants)
+            create_disposition: optional string
+                    (one of the JOB_CREATE_* constants)
+            write_disposition: optional string
+                    (one of the JOB_WRITE_* constants)
+
+            Optional arguments with value None are determined by
+            BigQuery as described:
+            https://developers.google.com/bigquery/docs/reference/v2/jobs
+
+        Returns:
+            dict, a BigQuery job resource
+        Raises:
+            JobInsertException on http/auth failures or error in result
+        """
+
+        configuration = {
+            "query": query,
+        }
+
+        if dataset and table:
+            configuration['destinationTable'] = {
+                "projectId": self.project_id,
+                "tableId": table,
+                "datasetId": dataset
+            }
+
+        if allow_large_results is not None:
+            configuration['allowLargeResults'] = allow_large_results
+
+        if use_query_cache is not None:
+            configuration['useQueryCache'] = use_query_cache
+
+        if priority:
+            configuration['priority'] = priority
+
+        if create_disposition:
+            configuration['createDisposition'] = create_disposition
+
+        if write_disposition:
+            configuration['writeDisposition'] = write_disposition
+
+        body = {
+            "configuration": {
+                'query': configuration
+            }
+        }
+
+        logging.info("Creating write to table job %s" % body)
+        job_resource = self.bigquery.jobs() \
+            .insert(projectId=self.project_id, body=body) \
+            .execute()
+        self._raise_insert_exception_if_error(job_resource)
+        return job_resource
 
     def wait_for_job(self, job, interval=5, timeout=None):
         """
         Waits until the job indicated by job_resource is done or has failed
         Args:
-            job: dict, representing a BigQuery job resource or jobId
+            job: dict, representing a BigQuery job resource
             interval: optional float polling interval in seconds, default = 5
             timeout: optional float timeout in seconds, default = None
         Returns:
             dict, final state of the job_resource, as described here:
-            https://developers.google.com/resources/api-libraries/documentation/bigquery/v2/python/latest/bigquery_v2.jobs.html#get
+            https://developers.google.com/resources/api-libraries/documentation
+            /bigquery/v2/python/latest/bigquery_v2.jobs.html#get
         Raises:
-            standard exceptions on http / auth failures (you must retry)
+            JobExecutingException on http/auth failures or error in result
         """
-        if isinstance(job, dict):  # job is a job resource
-            complete = job.get('jobComplete')
-            job_id = job['jobReference']['jobId']
-        else:  # job is the jobId
-            complete = False
-            job_id = job
-            job_resource = None
+        complete = False
+        job_id = job['jobReference']['jobId']
+        job_resource = None
 
         start_time = time()
         elapsed_time = 0
@@ -499,10 +705,7 @@ class BigQueryClient(object):
             request = self.bigquery.jobs().get(projectId=self.project_id,
                                                jobId=job_id)
             job_resource = request.execute()
-            error = job_resource.get('error')
-            if error:
-                raise Exception("{message} ({code}). Errors: {errors}",
-                                **error)
+            self._raise_executing_exception_if_error(job_resource)
             complete = job_resource.get('status').get('state') == u'DONE'
             elapsed_time = time() - start_time
 
@@ -518,7 +721,8 @@ class BigQueryClient(object):
             insert_id_key: key for insertId in row
 
         Returns:
-            bool indicating if insert succeeded or not.
+            bool indicating if insert succeeded or not, or response
+            from BigQuery if swallow_results is set for False.
         """
 
         table_data = self.bigquery.tabledata()
@@ -545,14 +749,26 @@ class BigQueryClient(object):
             ).execute()
 
             if response.get('insertErrors'):
-                logger.error('BigQuery insert errors: %s' % response)
+                logging.error('BigQuery insert errors: %s' % response)
+                if self.swallow_results:
+                    return False
+                else:
+                    return response
+
+            if self.swallow_results:
+                return True
+            else:
+                return response
+
+        except HttpError as e:
+            logging.exception('Problem with BigQuery insertAll')
+            if self.swallow_results:
                 return False
-
-            return True
-
-        except:
-            logger.error('Problem with BigQuery insertAll')
-            return False
+            else:
+                return {'insertErrors': [{
+                    'errors': [{'reason': 'httperror',
+                                'message': e.message}]
+                }]}
 
     def _get_all_tables(self, dataset_id):
         """Retrieve a list of all tables for the dataset.
@@ -670,8 +886,8 @@ class BigQueryClient(object):
         ONE_MONTH = 2764800  # 32 days
 
         return start_time <= time <= end_time or \
-               time <= start_time <= time + ONE_MONTH or \
-               time <= end_time <= time + ONE_MONTH
+            time <= start_time <= time + ONE_MONTH or \
+            time <= end_time <= time + ONE_MONTH
 
     def _get_query_results(self, job_collection, project_id, job_id,
                            offset=None, limit=None):
@@ -762,10 +978,41 @@ class BigQueryClient(object):
 
         return row_value
 
+    def _generate_hex_for_uris(self, uris):
+        """Given uris, generate and return hex version of it
+
+        Args:
+            uris: A list containing all uris
+        Returns:
+            string of hexed uris
+        """
+        return sha256(":".join(uris) + str(time())).hexdigest()
+
+    def _raise_insert_exception_if_error(self, job):
+        error_http = job.get('error')
+        if error_http:
+            raise JobInsertException(
+                "Error in export job API request: {0}".format(error_http))
+        # handle errorResult - API request is successful but error in result
+        error_result = job.get('status').get('errorResult')
+        if error_result:
+            raise JobInsertException(
+                "Reason:{reason}. Message:{message}".format(**error_result))
+
+    def _raise_executing_exception_if_error(self, job):
+        error_http = job.get('error')
+        if error_http:
+            raise JobExecutingException(
+                "Error in export job API request: {0}".format(error_http))
+        # handle errorResult - API request is successful but error in result
+        error_result = job.get('status').get('errorResult')
+        if error_result:
+            raise JobExecutingException(
+                "Reason:{reason}. Message:{message}".format(**error_result))
+
     #
     # DataSet manipulation methods
     #
-
     def create_dataset(self, dataset_id, friendly_name=None, description=None,
                        access=None):
         """Create a new BigQuery dataset.
@@ -781,7 +1028,8 @@ class BigQueryClient(object):
                     datasets#resource)
 
         Returns:
-            bool indicating if dataset was created or not
+            bool indicating if dataset was created or not, or response
+            from BigQuery if swallow_results is set for False
         """
         try:
             datasets = self.bigquery.datasets()
@@ -790,12 +1038,19 @@ class BigQueryClient(object):
                                                  description=description,
                                                  access=access)
 
-            datasets.insert(projectId=self.project_id,
+            response = datasets.insert(projectId=self.project_id,
                             body=dataset_data).execute()
-            return True
-        except Exception, e:
-            logger.error('Cannot create dataset %s, %s' % (dataset_id, e))
-            return False
+            if self.swallow_results:
+                return True
+            else:
+                return response
+        except HttpError as e:
+            logging.error('Cannot create dataset {0}, {1}'.format(dataset_id,
+                                                                  e))
+            if self.swallow_results:
+                return False
+            else:
+                return {}
 
     def get_datasets(self):
         """List all datasets in the project.
@@ -807,19 +1062,22 @@ class BigQueryClient(object):
             datasets = self.bigquery.datasets()
             request = datasets.list(projectId=self.project_id)
             result = request.execute()
-            return result
-        except Exception, e:
-            logger.error("Cannot list datasets: %s" % e)
+            return result.get('datasets', [])
+        except HttpError as e:
+            logging.error("Cannot list datasets: {0}".format(e))
             return None
 
-    def delete_dataset(self, dataset_id):
+    def delete_dataset(self, dataset_id, delete_contents=False):
         """Delete a BigQuery dataset.
 
         Args:
             dataset_id: required unique string identifying the dataset with the
                         project (the referenceId of the dataset)
+            delete_contents: forces deletion of the dataset even when the
+                        dataset contains data
         Returns:
-            bool indicating if the delete was successful or not
+            bool indicating if the delete was successful or not, or response
+            from BigQuery if swallow_results is set for False
 
         Raises:
             HttpError 404 when dataset with dataset_id does not exist
@@ -827,12 +1085,20 @@ class BigQueryClient(object):
         try:
             datasets = self.bigquery.datasets()
             request = datasets.delete(projectId=self.project_id,
-                                      datasetId=dataset_id)
-            request.execute()
-            return True
-        except Exception, e:
-            logger.error('Cannot delete dataset %s: %s' % (dataset_id, e))
-            return None
+                                      datasetId=dataset_id,
+                                      deleteContents=delete_contents)
+            response = request.execute()
+            if self.swallow_results:
+                return True
+            else:
+                return response
+        except HttpError as e:
+            logging.error('Cannot delete dataset {0}: {1}'.format(dataset_id,
+                                                                  e))
+            if self.swallow_results:
+                return False
+            else:
+                return {}
 
     def update_dataset(self, dataset_id, friendly_name=None, description=None,
                        access=None):
@@ -848,7 +1114,8 @@ class BigQueryClient(object):
             access: an optional object indicating access permissions.
 
         Returns:
-            bool indicating if the update was successful or not.
+            bool indicating if the update was successful or not, or response
+            from BigQuery if swallow_results is set for False.
         """
         try:
             datasets = self.bigquery.datasets()
@@ -857,11 +1124,18 @@ class BigQueryClient(object):
             request = datasets.update(projectId=self.project_id,
                                       datasetId=dataset_id,
                                       body=body)
-            request.execute()
-            return True
-        except Exception, e:
-            logger.error('Cannot update dataset %s: %s' % (dataset_id, e))
-            return False
+            response = request.execute()
+            if self.swallow_results:
+                return True
+            else:
+                return response
+        except HttpError as e:
+            logging.error('Cannot update dataset {0}: {1}'.format(dataset_id,
+                                                                  e))
+            if self.swallow_results:
+                return False
+            else:
+                return {}
 
     def patch_dataset(self, dataset_id, friendly_name=None, description=None,
                       access=None):
@@ -876,7 +1150,8 @@ class BigQueryClient(object):
             description: an optional description of the dataset.
             access: an optional object indicating access permissions.
         Returns:
-            bool indicating if the patch was successful or not.
+            bool indicating if the patch was successful or not, or response
+            from BigQuery if swallow_results is set for False.
         """
         try:
             datasets = self.bigquery.datasets()
@@ -884,11 +1159,17 @@ class BigQueryClient(object):
                                          description, access)
             request = datasets.patch(projectId=self.project_id,
                                      datasetId=dataset_id, body=body)
-            request.execute()
-            return True
-        except Exception, e:
-            logger.error('Cannot patch dataset %s: %s' % (dataset_id, e))
-            return False
+            response = request.execute()
+            if self.swallow_results:
+                return True
+            else:
+                return response
+        except HttpError as e:
+            logging.error('Cannot patch dataset {0}: {1}'.format(dataset_id, e))
+            if self.swallow_results:
+                return False
+            else:
+                return {}
 
     def dataset_resource(self, ref_id, friendly_name=None, description=None,
                          access=None):
